
"""
This file implements the InstantlabApi class which provides the API calls to
the middlware as methods.
"""

import re
import requests

from .factories import ModelFactory, ExceptionFactory
from .models import InfrastructureLayout, Infrastructure

API_VERSION = '0.2'


class InstantlabApi(object):
    """
    API client for the Instantlab REST API.

    Names are according to
    http://en.wikipedia.org/wiki/URI_scheme#Generic_syntax
    """

    def __init__(self, base_url, api_id, api_key):
        """
        Constructor.

        :param base_url: the base url of the middleware instance to connect to
        :param api_id: the auth id for the middleware
        :param api_key: the auth key for the middleware
        """
        # add schema, if missing
        if re.match('[a-zA-Z][a-zA-Z]*://', base_url) is None:
            base_url = "http://" + base_url

        self.base_url = base_url + '/api/' + str(API_VERSION) + "/"
        self.api_id = api_id
        self.api_key = api_key

        self.exception_factory = ExceptionFactory()
        self.infrastructure_layout_factory = ModelFactory(InfrastructureLayout)
        self.infrastructure_factory = ModelFactory(Infrastructure)

    def _request(self, path, user_id, data=None, method='GET'):
        """
        Issue a specific HTTP request.

        :param path: the path to append to the base url
        :param user_id: the user identifier
        :param data: the POST data to add to the request, if any
        :param method: the HTTP method to use, defaults to GET/POST

        :rtype: the json decoded response.
        """

        headers = {
            'X-API-ID': self.api_id,
            'X-API-Key': self.api_key,
        }

        if user_id is not None:
            headers['X-User-ID'] = user_id

        if data is not None:
            method = 'POST'

        res = requests.request(method, self.base_url + path, headers=headers, data=data)

        if res.status_code != 200:
            raise self.exception_factory.make(res.text)

        return res.json()['data']

    def _get(self, path, user_id=None):
        """
        Issue a GET request to the given path.

        :param path: the path to append to the base url
        :param user_id: the user identifier

        :rtype: the json decoded response.
        """
        return self._request(path, user_id)

    def _post(self, path, user_id=None, **data):
        """
        Issue a POST request to the given path.

        :param path: the path to append to the base url
        :param user_id: the user identifier
        :param \*\*data: the POST data to add to the request

        :rtype: the json decoded response.
        """
        return self._request(path, user_id, data)

    def _delete(self, path, user_id=None):
        """
        Issue a DELETE request to the given path.

        :param path: the path to append to the base url
        :param user_id: the user identifier

        :rtype: the json decoded response.
        """
        return self._request(path, user_id, method='DELETE')

    def list_infrastructure_layouts(self):
        """
        Get a list of infrastructure_layouts.

        :param user_id: optional. limit the search space to the instances visible by user_id

        :rtype: a list of infrastructure_layouts
        """
        response = self._get('infrastructure_layout/')
        return self.infrastructure_layout_factory.make_lazy_list(response, _api=self)

    def get_infrastructure_layout(self, infrastructure_layout_id):
        """
        Get details of an infrastructure_layout by its id.

        :param infrastructure_layout_id: the queried id
        :param user_id: optional. limit the search space to the instances visible by user_id

        :rtype: an instance of instantlab_apiclient.models.InfrastructureLayout
        """
        response = self._get('infrastructure_layout/%s/' % infrastructure_layout_id)
        return self.infrastructure_layout_factory.make(response)

    def list_infrastructures(self, user_id=''):
        """
        Get a list of infrastructures.

        :param user_id: optional. limit the returned list to the instances visible by user_id

        :rtype: a list of infrastructure ids
        """
        response = self._get('infrastructure/', user_id)
        return self.infrastructure_factory.make_lazy_list(response, _api=self, _user_id=user_id)

    def get_infrastructure(self, infrastructure_id, user_id=''):
        """
        Get details of an infrastructure by its identifier.

        :param infrastructure_id: the queried id
        :param user_id: optional. limit the search space to the instances visible by user_id

        :rtype: an instance of instantlab_apiclient.models.Infrastructure
        """
        response = self._get('infrastructure/%s/' % infrastructure_id, user_id)
        return self.infrastructure_factory.make(response)

    def create_infrastructure(self, infrastructure_layout_id, user_id=''):
        """
        Create an infrastructure from a layout and returns its identifier.

        :param infrastructure_layout_id: the queried id
        :param user_id: optional. limit the search space to the instances visible by user_id

        :rtype: an instance of instantlab_apiclient.models.Infrastructure
        """
        response = self._post('infrastructure/', user_id,
                              infrastructure_layout_id=infrastructure_layout_id)
        return self.infrastructure_factory.make_lazy(response, _api=self, _user_id=user_id)

    def delete_infrastructure(self, infrastructure_id, user_id=''):
        """
        Stop and delete an infrastructure by its identifier.

        :param infrastructure_id: the queried id
        :param user_id: optional. limit the search space to the instances visible by user_id
        """
        self._delete('infrastructure/%s/' % infrastructure_id, user_id)
