
"""
This file implements factories for the InstantlabApi to produce instances from
HTTP responses.
"""

from .exceptions import *


class ModelFactory(object):
    """
    A factory to produce lazy-load instances of a Model.
    """

    def __init__(self, cls):
        """
        Constructor.

        :param cls: the model class to produce
        :param api: a backref to an instance of InstantlabApi
        """
        self.cls = cls

    def make(self, data):
        """
        Create an instance of a Model from a dict containing its data.

        :param data: a dict containing the attributes of the instance

        :rtype: an instance of self.cls
        """
        return self.cls(**data)

    def make_lazy(self, data, **kwargs):
        """
        Create a lazy instance of a Models from an id.

        :param data: the id of a Model

        :rtype: a lazy instance of a Model
        """
        return self.cls(id=data, **kwargs)

    def make_lazy_list(self, data, **kwargs):
        """
        Create a list of lazy Models from a list of ids.

        :param data: a list of Model ids

        :rtype: a list of lazy Models
        """
        return [self.make_lazy(id, **kwargs) for id in data]


class ExceptionFactory(object):
    """
    A factory to create exception instances from 500 responses of the
    middleware.
    """

    def make(self, response):
        """
        Produce an exception from the given response instance (wip)

        :param response: the HTTPError produced by the failed call to the
                         middleware
        """
        return InstantlabApiError(repr(response))
