
"""
This file implements the models returned by the InstantlabApi.
"""


class Model(object):
    """
    The base model.
    """

    def __init__(self, **attrs):
        """
        Constructor. Sets all passed kwargs as attributes of the instance.
        This allows for lazy instances to be created with id and an api
        reference, while nonlazy instances can be created with full data.

        :param \*\*attrs: the attributes to be set on the instance
        """
        self._setattrs(attrs)

    def _setattrs(self, attrs):
        """
        Set attributes on the instance from a given dict. This is used to fill
        instances with data either from the constructor or during lazy-load
        time.

        :param attrs: a dict of attributes to be set on the instance
        """
        for key in attrs:
            setattr(self, key, attrs[key])

    def __getattr__(self, key):
        """
        A custom __getattr__ to perform the lazy-load. If an unknown attribute
        is queried, and an api reference is available, the data is queried
        from the api server.

        :param key: the queried key

        :rtype: the associated value
        """
        if '_api' not in self.__dict__:
            raise AttributeError("'%s' object has no attribute '%s'" % (type(self).__name__, key))

        self._load()

        return getattr(self, key)

    def _load(self):
        raise NotImplementedError


class InfrastructureLayout(Model):
    """
    A representation of an InfrastructureLayout.
    """

    def __setattr__(self, key, value):
        """
        A custom __setattr__ to convert the node templates dicts to
        NodeTemplate instances on load time.

        :param key: the queried key
        :param value: the value to be set
        """
        if key == 'node_templates':
            value = [NodeTemplate(**n) for n in value]
        super(InfrastructureLayout, self).__setattr__(key, value)

    def _load(self):
        """
        simple proxy function to the api data fetch routine
        """
        self._setattrs(self._api.get_infrastructure_layout(self.id).__dict__)
        del self._api


class Infrastructure(Model):
    """
    A representation of an Infrastructure.
    """

    def __setattr__(self, key, value):
        """
        A custom __setattr__ to convert the node dicts to Node instances on
        load time.

        :param key: the queried key
        :param value: the value to be set
        """
        if key == 'nodes':
            value = [Node(**n) for n in value]
        super(Infrastructure, self).__setattr__(key, value)

    def _load(self):
        """
        simple proxy function to the api data fetch routine
        """
        self._setattrs(self._api.get_infrastructure(self.id, self._user_id).__dict__)
        del self._api
        del self._user_id


class NodeTemplate(Model):
    """
    A representation of a NodeTemplate.
    """


class Node(Model):
    """
    A representation of a Node.
    """
